#pragma semicolon 1

#include <sourcemod>
#include <sdktools>
#include <rankme>
#include <morecolors>

#define PLUGIN_VERSION "1.2"

new Handle:h_Skin1, String:s_Skin1[PLATFORM_MAX_PATH];
new Handle:h_Skin2, String:s_Skin2[PLATFORM_MAX_PATH];
new Handle:h_Ranks, i_ranks;

new bool:b_PlayerHasSkin[MAXPLAYERS+1];

public Plugin:myinfo =
{
	name = "[RankMe] Rank skins",
	author = "FrozDark (HLModders.ru LLC)",
	description = "RankMe Plugin",
	version = PLUGIN_VERSION,
	url = "http://www.hlmod.ru/"
};

public APLRes:AskPluginLoad2(Handle:myself, bool:late, String:error[], err_max)
{
	return APLRes_Success;
}

public OnPluginStart()
{
	CreateConVar("rankme_skin_version", PLUGIN_VERSION, "RankMe rank skins version", FCVAR_PLUGIN|FCVAR_SPONLY|FCVAR_REPLICATED|FCVAR_NOTIFY|FCVAR_CHEAT|FCVAR_DONTRECORD);
	
	h_Ranks = CreateConVar("rankme_skin_ranks", "10", "How high to start rankme rank skins from", 0, true, 0.0, true, 10.0);
	h_Skin1 = CreateConVar("rankme_skin_team_1", "", "Skin for the team index 2. <\"T\", \"RED\", \"USA\">");
	h_Skin2 = CreateConVar("rankme_skin_team_2", "", "Skin for the team index 3. <\"CT\", \"BLU\", \"GER\">");
	
	i_ranks = GetConVarInt(h_Ranks);
	GetConVarString(h_Skin1, s_Skin1, sizeof(s_Skin1));
	GetConVarString(h_Skin2, s_Skin2, sizeof(s_Skin2));
	
	HookConVarChange(h_Ranks, ConVarChanges);
	HookConVarChange(h_Skin1, ConVarChanges);
	HookConVarChange(h_Skin2, ConVarChanges);
	
	HookEvent("player_spawn", Event_PlayerSpawn);
	HookEvent("player_team", Event_PlayerSpawn);
	HookEvent("round_start", Event_RoundStart);
	
	for (new client = 1; client <= MaxClients; client++)
	{
		if (IsClientInGame(client) && !IsFakeClient(client))
		{
			RankMe_GetRank(client, StatsCallback, 0);
		}
	}
	
	LoadTranslations("rankme_skins");
	
	AutoExecConfig(true, "rankme_skins");
}

public OnMapStart()
{
	decl String:buffer[PLATFORM_MAX_PATH];
	BuildPath(Path_SM, buffer, sizeof(buffer), "configs/rankme_dlist.ini");
	
	File_ReadDownloadList(buffer);
}

public OnConfigsExecuted()
{
	if (s_Skin1[0] && IsModelFile(s_Skin1))
	{
		PrecacheModel(s_Skin1, true);
	}
	else
	{
		s_Skin1[0] = '\0';
	}
	if (s_Skin2[0] && IsModelFile(s_Skin2))
	{
		PrecacheModel(s_Skin2, true);
	}
	else
	{
		s_Skin2[0] = '\0';
	}
}

public OnClientPutInServer(client)
{
	if (IsFakeClient(client) || i_ranks < 1)
		return;
	
	RankMe_GetRank(client, StatsCallback, 0);
}

public StatsCallback(client, rank, any:data)
{
	if (!client || !IsClientInGame(client))
	{
		return;
	}
	if (0 < rank <= i_ranks)
	{
		if (!b_PlayerHasSkin[client])
		{
			CPrintToChat(client, "%t", "InTop", rank, i_ranks);
		}
			
		b_PlayerHasSkin[client] = true;
		CreateTimer(0.1, SetModel, GetClientUserId(client), TIMER_FLAG_NO_MAPCHANGE);
	}
	else if (b_PlayerHasSkin[client])
	{
		b_PlayerHasSkin[client] = false;
		CPrintToChat(client, "%t", "OutTop", i_ranks);
	}
}

public Action:SetModel(Handle:timer, any:userid)
{
	new client = GetClientOfUserId(userid);
	
	if (i_ranks < 1 || !client || !b_PlayerHasSkin[client] || !IsPlayerAlive(client))
	{
		return;
	}

	switch (GetClientTeam(client))
	{
		case 2 :
		{
			if (s_Skin1[0])
			{
				SetEntityModel(client, s_Skin1);
			}
		}
		case 3 :
		{
			if (s_Skin2[0])
			{
				SetEntityModel(client, s_Skin2);
			}
		}
	}
}

public Event_PlayerSpawn(Handle:event, const String:name[], bool:dontBroadcast)
{
	new userid = GetEventInt(event, "userid");
	CreateTimer(0.1, SetModel, userid, TIMER_FLAG_NO_MAPCHANGE);
}

public Event_RoundStart(Handle:event, const String:name[], bool:dontBroadcast)
{
	if (i_ranks < 1)
	{
		return;
	}
	for (new client = 1; client <= MaxClients; client++)
	{
		if (IsClientInGame(client) && !IsFakeClient(client))
		{
			RankMe_GetRank(client, StatsCallback, 0);
		}
	}
}

// ======================================================================================================================================================

public ConVarChanges(Handle:convar, const String:oldValue[], const String:newValue[])
{
	if (convar == h_Ranks)
	{
		i_ranks = StringToInt(newValue);
	}
	else if (convar == h_Skin1)
	{
		strcopy(s_Skin1, sizeof(s_Skin1), newValue);
		if (s_Skin1[0] && IsModelFile(s_Skin1))
		{
			PrecacheModel(s_Skin1);
		}
		else
		{
			s_Skin1[0] = '\0';
		}
	}
	else if (convar == h_Skin2)
	{
		strcopy(s_Skin2, sizeof(s_Skin2), newValue);
		if (s_Skin2[0] && IsModelFile(s_Skin2))
		{
			PrecacheModel(s_Skin2);
		}
		else
		{
			s_Skin2[0] = '\0';
		}
	}
}

// ======================================================================================================================================================
new String:_smlib_empty_twodimstring_array[][] = { { '\0' } };
stock File_AddToDownloadsTable(const String:path[], bool:recursive=true, const String:ignoreExts[][]=_smlib_empty_twodimstring_array, size=0)
{
	if (path[0] == '\0') {
		return;
	}

	if (FileExists(path)) {
		
		new String:fileExtension[4];
		File_GetExtension(path, fileExtension, sizeof(fileExtension));
		
		if (StrEqual(fileExtension, "bz2", false) || StrEqual(fileExtension, "ztmp", false)) {
			return;
		}
		
		if (Array_FindString(ignoreExts, size, fileExtension) != -1) {
			return;
		}

		AddFileToDownloadsTable(path);
	}
	else if (recursive && DirExists(path)) {

		decl String:dirEntry[PLATFORM_MAX_PATH];
		new Handle:__dir = OpenDirectory(path);

		while (ReadDirEntry(__dir, dirEntry, sizeof(dirEntry))) {

			if (StrEqual(dirEntry, ".") || StrEqual(dirEntry, "..")) {
				continue;
			}
			
			Format(dirEntry, sizeof(dirEntry), "%s/%s", path, dirEntry);
			File_AddToDownloadsTable(dirEntry, recursive, ignoreExts, size);
		}
		
		CloseHandle(__dir);
	}
	else if (FindCharInString(path, '*', true)) {
		
		new String:fileExtension[4];
		File_GetExtension(path, fileExtension, sizeof(fileExtension));

		if (StrEqual(fileExtension, "*")) {

			decl
				String:dirName[PLATFORM_MAX_PATH],
				String:fileName[PLATFORM_MAX_PATH],
				String:dirEntry[PLATFORM_MAX_PATH];

			File_GetDirName(path, dirName, sizeof(dirName));
			File_GetFileName(path, fileName, sizeof(fileName));
			StrCat(fileName, sizeof(fileName), ".");

			new Handle:__dir = OpenDirectory(dirName);
			while (ReadDirEntry(__dir, dirEntry, sizeof(dirEntry))) {

				if (StrEqual(dirEntry, ".") || StrEqual(dirEntry, "..")) {
					continue;
				}

				if (strncmp(dirEntry, fileName, strlen(fileName)) == 0) {
					Format(dirEntry, sizeof(dirEntry), "%s/%s", dirName, dirEntry);
					File_AddToDownloadsTable(dirEntry, recursive, ignoreExts, size);
				}
			}

			CloseHandle(__dir);
		}
	}

	return;
}

stock File_ReadDownloadList(const String:path[])
{
	new Handle:file = OpenFile(path, "r");
	
	if (file  == INVALID_HANDLE) {
		return;
	}

	new String:buffer[PLATFORM_MAX_PATH];
	while (!IsEndOfFile(file)) {
		ReadFileLine(file, buffer, sizeof(buffer));
		
		new pos;
		pos = StrContains(buffer, "//");
		if (pos != -1) {
			buffer[pos] = '\0';
		}
		
		pos = StrContains(buffer, "#");
		if (pos != -1) {
			buffer[pos] = '\0';
		}

		pos = StrContains(buffer, ";");
		if (pos != -1) {
			buffer[pos] = '\0';
		}
		
		TrimString(buffer);
		
		if (buffer[0] == '\0') {
			continue;
		}

		File_AddToDownloadsTable(buffer);
	}

	CloseHandle(file);
}

stock File_GetExtension(const String:path[], String:buffer[], size)
{
	new extpos = FindCharInString(path, '.', true);
	
	if (extpos == -1)
	{
		buffer[0] = '\0';
		return;
	}

	strcopy(buffer, size, path[++extpos]);
}

stock Array_FindString(const String:array[][], size, const String:str[], bool:caseSensitive=true, start=0)
{
	if (start < 0) {
		start = 0;
	}

	for (new i=start; i < size; i++) {

		if (StrEqual(array[i], str, caseSensitive)) {
			return i;
		}
	}
	
	return -1;
}

stock bool:File_GetFileName(const String:path[], String:buffer[], size)
{	
	if (path[0] == '\0') {
		buffer[0] = '\0';
		return;
	}
	
	File_GetBaseName(path, buffer, size);
	
	new pos_ext = FindCharInString(buffer, '.', true);

	if (pos_ext != -1) {
		buffer[pos_ext] = '\0';
	}
}

stock bool:File_GetDirName(const String:path[], String:buffer[], size)
{	
	if (path[0] == '\0') {
		buffer[0] = '\0';
		return;
	}
	
	new pos_start = FindCharInString(path, '/', true);
	
	if (pos_start == -1) {
		pos_start = FindCharInString(path, '\\', true);
		
		if (pos_start == -1) {
			buffer[0] = '\0';
			return;
		}
	}
	
	strcopy(buffer, size, path);
	buffer[pos_start] = '\0';
}

stock bool:File_GetBaseName(const String:path[], String:buffer[], size)
{	
	if (path[0] == '\0') {
		buffer[0] = '\0';
		return;
	}
	
	new pos_start = FindCharInString(path, '/', true);
	
	if (pos_start == -1) {
		pos_start = FindCharInString(path, '\\', true);
	}
	
	pos_start++;
	
	strcopy(buffer, size, path[pos_start]);
}

bool:IsModelFile(const String:model[])
{
	decl String:buf[4];
	File_GetExtension(model, buf, sizeof(buf));
	if (!strcmp(buf, "mdl", false))
	{
		return true;
	}
	
	return false;
}
// ======================================================================================================================================================